#!/bin/sh

PKG_STR_WIZARD_UPGRADE_TITLE="Read before proceeding"
PKG_STR_WIZARD_UPGRADE_NOTE="Upgrading to Surveillance Station requires database conversion and thus may take a while, depending on the number of recordings in the database. Please wait patiently and do not power off the system or uninstall the package during this process."
PKG_STR_WIZARD_UPGRADE_VS_60_80_NOTE="This version of Surveillance Station is only compatible with VisualStation {0} and onward. VisualStation VS60 and VS80 models are no longer supported by this version of Surveillance Station."
PKG_STR_WIZARD_UPGRADE_VS_360HD_NOTE="After VS360HD is upgraded to version 4.0 and onwards, the original pairing function of Visual Station will be replaced by the new login mechanism. You will be able to log in to Surveillance Station directly from your Synology VisualStation VS360HD. Moreover, all the network video recording features powered by Surveillance Station are going to be supported on VS360HD.<br><br>The current version of Surveillance Station is compatible with both original pairing function of VisualStation version 3.0.5 and new login mechanism of VisualStation version 4.0. However, the original pairing function will end in the upcoming updates."
PKG_STR_WIZARD_UPGRADE_PSIA_NOTE="Surveillance Station no longer supports PSIA since this version. You are unable to use the cameras which have been set up with PSIA in Surveillance Station."
PKG_STR_WIZARD_INCOMPATIBLE_CHECK_LABEL="I understand the compatibility restrictions, and wish to proceed with the update."
PKG_STR_WIZARD_UPGRADE_DOOR_LICENSE="This version of Surveillance Station will apply new license policy to access controllers.<br><br>Each newly added access controller will be counted as one surveillance device, and will use a surveillance device license as do IP cameras and I/O modules. However, your previously installed access controllers can still be used without licenses after this update."
PKG_STR_WIZARD_DOOR_LICENSE_CHECK_LABEL="I understand the new license policy for access controllers, and wish to proceed with the update."
PKG_STR_WIZARD_DOOR_PRIVILEGE_CHECK_NOTE="After upgrading to this version, privilege management for door controllers will be moved to Privilege Profile in the Users page."
PKG_STR_WIZARD_UPGRADE_LABEL="I understand the changes that will be made, and agree to proceed with the update."
PKG_STR_WIZARD_EDS14_EOL="Surveillance Station 8.2.9 no longer supports EDS14. It will be removed from CMS after the update."
PKG_STR_WIZARD_SUGGEST_X64_HOST_NOTE="Because CMS supports Face Recognition and Deep Video Analytics functions, we recommend using an x64 server as the CMS Host to ensure a better user experience."
PKG_STR_WIZARD_FACE_DATABASE_MERGE="This update adds the support for Face Recognition in a CMS structure. Face Databases on DVA servers will be merged into one. (<a href='http://sy.to/4umx3' target='_blank'>Merge Rules</a>)"
PKG_STR_WIZARD_REMOVE_DEEP_MD="This update removes Deep Motion Detection. Existing Deep Motion Detection tasks will be migrated into People and Vehicle Detection tasks while keeping their original settings."
PKG_STR_PKG_UPGRADE_AME_NOTE="Due to updates in Advanced Media Extensions' support mechanism, this version will make the following changes:<br><ul style={0}margin-left:20px;list-style:circle;{0}><li>For cameras that use AAC audio format:<ul style={0}list-style:square inside;{0}><li>If the cameras support other audio formats, another format will be applied.</li><li>If the cameras do not support other audio formats, Surveillance Station can no longer play the audio of their videos. You need to download the recordings for the sound.</li></ul></li><li>For cameras that use H.265 video format: <ul style={0}list-style:square inside;{0}><li>Motion Detection by Surveillance Station will no longer be available.</li><li>No thumbnails will be shown in video lists and event notifications.</li><li>Surveillance Station can no longer play their videos. You need to download the recordings for viewing.</li></ul></li></ul> If you wish to use the features above, go to the <b>Advanced Media Extensions</b> package for further operations."

# Copyright (c) 2021 Synology Inc. All rights reserved.

SS_PKG_PATH="/var/packages/SurveillanceStation/"
SS_INFO_FILE=${SS_PKG_PATH}"INFO"
SS_SYSTEM_DB_FILE=${SS_PKG_PATH}"target/system.db"
SS_AXISDOOR_DB_FILE=${SS_PKG_PATH}"target/axisacsctrl.db"
SS_AXISDOORLOG_DB_FILE=${SS_PKG_PATH}"target/@surveillance/axisacsevtlog.db"
SS_VER_KEY="version"
SS_SQLITE3_VER="6.3"
SS_THUMBNAIL_ISSUE_BUILD_VER="3312"
SS_THUMBNAIL_FIXED_BUILD_VER="3352"
SS_ADD_DOORLOG_INDEX_BUILD_VER="5372"
VS_MODELS_60_80="'VS60', 'VS80'"
VS_MODELS_360HD="'VS360HD'"
VS_360HD_NEW_LOGIN_MECHANISM_MIN_VERSION="'4.0'"
SS_UNSUPPORT_VENDOR="('PSIA')"
SS_MIN_VS_60_80_VERSION="2.2-1413"
SS_MIN_VS_360_VERSION="3.1-2500"
SS_SQLITE3="${SS_PKG_PATH}target/bin/sqlite3"
DEF_SQLITE3="/usr/syno/bin/sqlite3"
SS_MIN_DOOR_LICENSE_VERSION="8.0.0-5090"
SS_MIN_DOOR_PRIVILEGE_VERSION="8.1.0-5323"
SS_MIN_DVA_SUPPORT_CMS_VERSION="8.2.9-6391"
CMS_DISABLED_FILE=${SS_PKG_PATH}"target/@SSData/AddOns/CMS/disabled"
SS_SETTINGS_CONF_FILE=${SS_PKG_PATH}"etc/settings.conf"
SZK_CMS_MODE="ss_cms_mode"
SZV_CMS_HOST="host"
FN_CHECK_BOX_VALIDATOR="{ var blChecked = arguments[0]; return true === blChecked; }"
DEF_SYNOINFO_CONF="/etc.defaults/synoinfo.conf"
DSM_VERSION_FILE="/etc.defaults/VERSION"

${SS_SQLITE3} --version
if [ $? == 0 ]; then
	SQLITE3_BIN=$SS_SQLITE3
else
	${DEF_SQLITE3} --version
	if [ $? == 0 ]; then
		SQLITE3_BIN=$DEF_SQLITE3
	else
		SQLITE3_BIN="sqlite3"
	fi
fi

UPGRADE_VS_60_80_NOTE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "$(echo $PKG_STR_WIZARD_UPGRADE_VS_60_80_NOTE | sed "s/{0}/${SS_MIN_VS_60_80_VERSION}/g")",
		"subitems": [{
			"desc": "${PKG_STR_WIZARD_INCOMPATIBLE_CHECK_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			}
		}]
	}]
}
EOF`

UPGRADE_VS_360_NOTE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"items": [{
		"desc": "$PKG_STR_WIZARD_UPGRADE_VS_360HD_NOTE"
	}]
}
EOF`

UPGRADE_PSIA_NOTE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "${PKG_STR_WIZARD_UPGRADE_PSIA_NOTE}",
		"subitems": [{
			"desc": "${PKG_STR_WIZARD_INCOMPATIBLE_CHECK_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			}
		}]
	}]
}
EOF`

UPGRADE_DB_NOTE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"desc": "${PKG_STR_WIZARD_UPGRADE_NOTE}"
	}]
}
EOF`

UPGRADE_DOOR_LICENSE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "$PKG_STR_WIZARD_UPGRADE_DOOR_LICENSE",
		"subitems": [{
			"desc": "${PKG_STR_WIZARD_DOOR_LICENSE_CHECK_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			}
		}]
	}]
}
EOF`

UPGRADE_DOOR_PRIVILEGE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "${PKG_STR_WIZARD_DOOR_PRIVILEGE_CHECK_NOTE}",
		"subitems": [{
			"desc": "${PKG_STR_WIZARD_UPGRADE_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			}
		}]
	}]
}
EOF`

EDS14_EOL_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "${PKG_STR_WIZARD_EDS14_EOL}",
		"subitems": [{
			"key": "ui_confirmed_eds14_eol",
			"desc": "${PKG_STR_WIZARD_UPGRADE_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			},
			"defaultValue": false
		}]
	}]
}
EOF`

SUGGEST_X64_HOST_NOTE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"items": [{
		"type": "multiselect",
		"desc": "${PKG_STR_WIZARD_SUGGEST_X64_HOST_NOTE}",
		"subitems": [{
			"key": "ui_confirmed_suggest_x64_host",
			"defaultValue": true,
			"hidden": true
		}]
	}]
}
EOF`

FACE_DATABASE_MERGE_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "${PKG_STR_WIZARD_FACE_DATABASE_MERGE}",
		"subitems": [{
			"key": "ui_confirmed_face_database_merge",
			"desc": "${PKG_STR_WIZARD_UPGRADE_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			},
			"defaultValue": false
		}]
	}]
}
EOF`

REMOVE_DEEP_MD_PAGE=`/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"invalid_next_disabled": true,
	"items": [{
		"type": "multiselect",
		"desc": "${PKG_STR_WIZARD_REMOVE_DEEP_MD}",
		"subitems": [{
			"key": "ui_confirmed_remove_deep_md",
			"desc": "${PKG_STR_WIZARD_UPGRADE_LABEL}",
			"validator": {
				"fn": "$FN_CHECK_BOX_VALIDATOR"
			},
			"defaultValue": false
		}]
	}]
}
EOF`

Sqlite3Exec()
{
	$SQLITE3_BIN -init /dev/null -cmd ".timeout 300000" "$@"
}

IsUpgradeToSqlite3()
{
	local PreSSVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}" | cut -d '-' -f 1)

	if [ ${SS_SQLITE3_VER} \> ${PreSSVer} ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsUpgradeToDoorLicense()
{
	local TargetSSVer=$(echo ${SS_MIN_DOOR_LICENSE_VERSION} | sed 's/\.//g' | sed 's/-//g')
	local CurrentSSVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}")

	CurrentSSVer=$(echo ${CurrentSSVer} | sed 's/\.//g' | sed 's/-//g')

	if [ "${TargetSSVer}" -gt "${CurrentSSVer}" ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsUpgradeThumbnailToFile()
{
	local PreSSBuildVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}" | cut -d '-' -f 2)

	if [ ${SS_THUMBNAIL_ISSUE_BUILD_VER} -le ${PreSSBuildVer} ] && [ ${SS_THUMBNAIL_FIXED_BUILD_VER} -gt ${PreSSBuildVer} ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsUpgradeToDoorPrivilege()
{
	local TargetSSVer=$(echo ${SS_MIN_DOOR_PRIVILEGE_VERSION} | sed 's/\.//g' | sed 's/-//g')
	local CurrentSSVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}")

	CurrentSSVer=$(echo ${CurrentSSVer} | sed 's/\.//g' | sed 's/-//g')

	if [ "${TargetSSVer}" -ge "${CurrentSSVer}" ]; then
		echo "true"
	else
		echo "false"
	fi
}

HasVS()
{
	local Models="$1"
	local MinVersion="$2"
	local SqlCmd="SELECT COUNT(*) FROM visualstation WHERE platform IN ($Models)"

	if [ -n "$MinVersion" ]; then
		SqlCmd="SELECT COUNT(*) FROM visualstation WHERE platform IN ($Models) AND version < $MinVersion"
	else
		SqlCmd="SELECT COUNT(*) FROM visualstation WHERE platform IN ($Models)"
	fi

	CheckSystemDBItemExist "${SqlCmd}"
}

HasPsiaCam()
{
	local SqlCmd="SELECT COUNT(*) FROM camera WHERE vendor IN ${SS_UNSUPPORT_VENDOR}"

	CheckSystemDBItemExist "${SqlCmd}"
}

HasDoor()
{
	local DoorCount=$(sqlite3 $SS_AXISDOOR_DB_FILE "SELECT COUNT (id) FROM axisacsctrler where id IS NOT NULL")

	if [ $DoorCount -gt 0 ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsCmsHostHasEds14Slave()
{
	local CmsHost=$(get_key_value "${SS_SETTINGS_CONF_FILE}" "${SZK_CMS_MODE}")
	local Eds14RecCnt=$(sqlite3 ${SS_SYSTEM_DB_FILE} "SELECT count(id) FROM slaveds WHERE ds_model = 'EDS14'")

	if [ ! -f ${CMS_DISABLED_FILE} ] && [ ${SZV_CMS_HOST} == ${CmsHost} ] && [ ${Eds14RecCnt} -gt 0 ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsCmsHostHasDvaSlave()
{
	local CmsHost=$(get_key_value "${SS_SETTINGS_CONF_FILE}" "${SZK_CMS_MODE}")
	local DvaRecCnt=$(Sqlite3Exec "${SS_SYSTEM_DB_FILE}" "SELECT count(id) FROM slaveds WHERE ds_model like '%DVA%'")

	if [ ! -f ${CMS_DISABLED_FILE} ] && [ ${SZV_CMS_HOST} == ${CmsHost} ] && [ ${DvaRecCnt} -gt 0 ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsX64Arch()
{
	local Arch=$(/bin/get_key_value "${SS_INFO_FILE}" "arch")

	if [ "${Arch}" == "x86_64" ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsSupportSynoface()
{
	local UniqueName=$(/bin/get_key_value "${DEF_SYNOINFO_CONF}" "unique")
	if [[ "${UniqueName}" == "synology_denverton_dva3219" || "${UniqueName}" == "synology_denverton_dva3221" ]]; then
		echo "true"
	else
		echo "false"
	fi
}

IsMergeFaceDatabase()
{
	if [ -f ${CMS_DISABLED_FILE} ]; then
		echo "false"
		return 0
	fi

	local CmsMode=$(get_key_value "${SS_SETTINGS_CONF_FILE}" "${SZK_CMS_MODE}")
	local DvaRecCnt=$(Sqlite3Exec "${SS_SYSTEM_DB_FILE}" "SELECT count(id) FROM slaveds WHERE ds_model like '%DVA%'")

	if [ "false" == $(IsSupportSynoface) ]; then
		if [ ${SZV_CMS_HOST} != ${CmsMode} ] || [ ${DvaRecCnt} -le 0 ]; then
			echo "false"
			return 0
		fi
	else
		if [ ${SZV_CMS_HOST} == ${CmsMode} ] && [ ${DvaRecCnt} -le 0 ]; then
			echo "false"
			return 0
		fi
	fi

	echo "true"
}

IsHasDeepMD()
{
	local DeepMDCnt=$(Sqlite3Exec "${SS_SYSTEM_DB_FILE}" "SELECT count(id) FROM iva_setting WHERE analyze_type = 1")

	if [ "false" == $(IsSupportSynoface) ] || [ ${DeepMDCnt} -le 0 ]; then
		echo "false"
	else
		echo "true"
	fi
}

IsBeforeMinDvaSupportCmsVersion()
{
	local TargetSSVer=$(echo ${SS_MIN_DVA_SUPPORT_CMS_VERSION} | sed 's/\.//g' | sed 's/-//g')
	local CurrentSSVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}")

	CurrentSSVer=$(echo ${CurrentSSVer} | sed 's/\.//g' | sed 's/-//g')

	if [ "${TargetSSVer}" -ge "${CurrentSSVer}" ]; then
		echo "true"
	else
		echo "false"
	fi
}

HasLargeDoorLog()
{
	local PreSSBuildVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}" | cut -d '-' -f 2)
	local DoorLogCount=$(sqlite3 $SS_AXISDOORLOG_DB_FILE "SELECT COUNT (*) FROM axisacsevtlog")

	if [ ${SS_ADD_DOORLOG_INDEX_BUILD_VER} -gt ${PreSSBuildVer} ] && [ ${DoorLogCount} -gt 60000 ]; then
		echo "true"
	else
		echo "false"
	fi
}

CheckSystemDBItemExist()
{
	local SqlCmd="$1"
	local Cnt=$(Sqlite3Exec $SS_SYSTEM_DB_FILE "$SqlCmd")

	if [ -z "${Cnt}" ]; then
		if [ -f /usr/syno/pgsql/bin/psql ]; then
			Cnt=$(/usr/syno/pgsql/bin/psql -A -t -U postgres surveillance -c "${SqlCmd}")
		else
			Cnt=0
		fi
	fi

	if [ ${Cnt} -gt 0 ]; then
		echo "true"
	else
		echo "false"
	fi
}

IsUpgradeToAMEPlus()
{
	local DsmMajor=$(/bin/get_key_value "${DSM_VERSION_FILE}" "majorversion")

	if [ $DsmMajor -lt 7 ]; then
		echo "false"
		return
	fi

	local PrevSSVer=$(/bin/get_key_value "${SS_INFO_FILE}" "${SS_VER_KEY}")
	local PrevSSMajor=$(echo $PrevSSVer | cut -d '.' -f 1)
	local PrevSSBuildNum=$(echo $PrevSSVer | cut -d '-' -f 2)

	if [ $PrevSSMajor -lt 9 ]; then
		/bin/test $PrevSSBuildNum -lt 7320 && echo "true" || echo "false"
	else
		/bin/test $PrevSSBuildNum -lt 7600 && echo "true" || echo "false"
	fi
}

GET_AME_NOTE_PAGE()
{
/bin/cat<<EOF
{
	"step_title": "${PKG_STR_WIZARD_UPGRADE_TITLE}",
	"items": [{
		"desc": "$(echo "$PKG_STR_PKG_UPGRADE_AME_NOTE" | sed 's/{0}/\\"/g')"
	}]
}
EOF
}

GetUpgradeNotePages()
{
	local Pages=""

	if [ "true" == $(HasPsiaCam) ]; then
		Pages="${UPGRADE_PSIA_NOTE_PAGE}"
	fi

	if [ "true" == $(HasVS "$VS_MODELS_60_80") ]; then
		[ -z "$Pages" ] && Pages="${UPGRADE_VS_60_80_NOTE_PAGE}" \
						|| Pages="${Pages},${UPGRADE_VS_60_80_NOTE_PAGE}"
	fi

	if [ "true" == $(HasVS "$VS_MODELS_360HD" "$VS_360HD_NEW_LOGIN_MECHANISM_MIN_VERSION") ]; then
		[ -z "$Pages" ] && Pages="${UPGRADE_VS_360_NOTE_PAGE}" \
						|| Pages="${Pages},${UPGRADE_VS_360_NOTE_PAGE}"
	fi

	if [ "true" == $(IsUpgradeToSqlite3) ] || [ "true" == $(IsUpgradeThumbnailToFile) ] || [ "true" == $(HasLargeDoorLog) ]; then
		[ -z "$Pages" ] && Pages="${UPGRADE_DB_NOTE_PAGE}" \
						|| Pages="${Pages},${UPGRADE_DB_NOTE_PAGE}"
	fi

	if [ "true" == $(IsUpgradeToDoorLicense) ] && [ "true" == $(HasDoor) ]; then
		[ -z "$Pages" ] && Pages="${UPGRADE_DOOR_LICENSE_PAGE}" \
						|| Pages="${Pages},${UPGRADE_DOOR_LICENSE_PAGE}"
	fi

	if [ "true" == $(IsUpgradeToDoorPrivilege) ] && [ "true" == $(HasDoor) ]; then
		[ -z "$Pages" ] && Pages="${UPGRADE_DOOR_PRIVILEGE_PAGE}" \
						|| Pages="${Pages},${UPGRADE_DOOR_PRIVILEGE_PAGE}"
	fi

	if [ "true" == $(IsCmsHostHasEds14Slave) ]; then
		[ -z "$Pages" ] && Pages="${EDS14_EOL_PAGE}" \
						|| Pages="${Pages},${EDS14_EOL_PAGE}"
	fi

	if [ "true" == $(IsCmsHostHasDvaSlave) ] && [ "false" == $(IsX64Arch) ] && [ "true" == $(IsBeforeMinDvaSupportCmsVersion) ]; then
		[ -z "$Pages" ] && Pages="${SUGGEST_X64_HOST_NOTE_PAGE}" \
						|| Pages="${Pages},${SUGGEST_X64_HOST_NOTE_PAGE}"
	fi

	if [ "true" == $(IsHasDeepMD) ] && [ "true" == $(IsBeforeMinDvaSupportCmsVersion) ]; then
		[ -z "$Pages" ] && Pages="${REMOVE_DEEP_MD_PAGE}" \
						|| Pages="${Pages},${REMOVE_DEEP_MD_PAGE}"
	fi

	if [ "true" == $(IsMergeFaceDatabase) ] && [ "true" == $(IsBeforeMinDvaSupportCmsVersion) ]; then
		[ -z "$Pages" ] && Pages="${FACE_DATABASE_MERGE_PAGE}" \
						|| Pages="${Pages},${FACE_DATABASE_MERGE_PAGE}"
	fi

	if [ "true" == $(IsUpgradeToAMEPlus) ]; then
		[ -z "$Pages" ] && Pages="$(GET_AME_NOTE_PAGE)" \
						|| Pages="${Pages},$(GET_AME_NOTE_PAGE)"
	fi

	echo "[$Pages]"
}

main()
{
	local UpgradeNotePages="$(GetUpgradeNotePages)"

	echo "$UpgradeNotePages" > $SYNOPKG_TEMP_LOGFILE

	exit 0
}

main "$@"
