#!/bin/sh
# Copyright (c) 2022 Synology Inc. All rights reserved.

###############################################################
#   Usable environment variable:
#       PACK_STATUS, PACK_VERSION, PACK_NAME
#
#   Note: The script should bear to be invoked multiple times
###############################################################

SURVEILLANCE_INC_SCRP=/var/packages/SurveillanceStation/scripts/SurveillanceStationCommon
. ${SURVEILLANCE_INC_SCRP}

#At begining, acquire package settings
PackageInfoGet;

PreAMEUninstall()
{
	if [ false == $(IsSSCoredRunning) ]; then
		SSDebugLog "Ignore AME+ pre-uninstall, svs not running, ss_enabled=$(__IsSSEnabled).";
		return
	fi

	if [ true == $(IsUsingAME) ]; then
		test "UPGRADE" == "$PACK_STATUS" && touch "$CODECPACK_HOOK_AME_UPGRADE" || touch "$CODECPACK_HOOK_AME_UNINSTALL"

		SSDebugLog "AME+ pre-uninstall, stop svs, PACK_STATUS=$PACK_STATUS.";
		SurveillanceStationStop
	fi
}

PostAMEUninstall()
{
	if [ -f "$CODECPACK_HOOK_AME_UNINSTALL" ]; then
		rm -f "$CODECPACK_HOOK_AME_UNINSTALL"

		SSDebugLog "AME+ post-uninstall, restart svs.";
		SurveillanceStationRestart
	elif [ true == $(IsSSCoredRunning) ]; then
		SSDebugLog "AME+ post-uninstall, validate AME.";
		${S82_SCRIPT} trigger-ssctl validate-ame
	elif [ true == $(IsUsingAME) ]; then
		SSDebugLog "AME+ post-uninstall, change ffmpeg link to built-in.";
		LinkFFmpegCodecPack
	fi
}

case $1 in
	--sdk-mod-ver)
		#Print SDK support version
		echo "1.0"
	;;
	--name)
		#Print package name
		echo ${SURVEILLANCE_PKG_NAME};
	;;
	--pkg-ver)
		#Print package version
		echo ${SURVEILLANCE_PKG_VERSION};
	;;
	--vendor)
		#Print package vendor
		echo ${SURVEILLANCE_PKG_VENDOR};
	;;
	--app-id)
		#Print dsm app name
		echo ${SURVEILLANCE_DSM_APP_NAME};
	;;
	--pre)
		PreAMEUninstall
	;;
	--post)
		PostAMEUninstall
	;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--app-id|--pre|--post"
	;;
esac

